@@ -6,12 +6,13 @@ from undouble.undouble import (
     compute_blur,
     wget,
     unzip,
+    seperate_path,
 )
 
 
 __author__ = 'Erdogan Tasksen'
 __email__ = 'erdogant@gmail.com'
-__version__ = '1.1.1'
+__version__ = '1.2.0'
 
 # module level doc-string
 __doc__ = """
@@ -50,7 +51,7 @@ Example
 >>> model.import_data(targetdir)
 >>>
 >>> # Compute image-hash
->>> model.fit_transform()
+>>> model.compute_hash()
 >>>
 >>> # Group images with image-hash <= threshold
 >>> model.group(threshold=0)
