@@ -1,5 +1,5 @@
 # %%
-from undouble import Undouble
+# from undouble import Undouble
 # import undouble
 # print(dir(undouble))
 # print(undouble.__version__)
@@ -11,7 +11,7 @@ targetdir = 'D://magweg/101_ObjectCategories'
 from undouble import Undouble
 
 # Init with default settings
-model = Undouble(method='phash', hash_size=16)
+model = Undouble()
 
 # Import example data
 targetdir = model.import_example(data='flowers')
@@ -20,10 +20,10 @@ targetdir = model.import_example(data='flowers')
 model.import_data(targetdir)
 
 # Compute image-hash
-model.fit_transform(method='phash')
+model.compute_hash(method='phash', hash_size=16)
 
 # Find images with image-hash <= threshold
-model.group(threshold=0)
+model.group(threshold=5)
 
 # Plot the images
 model.plot()
@@ -31,6 +31,34 @@ model.plot()
 # Move the images
 # model.move()
 
+
+# %%
+from clustimage import Clustimage
+
+# Cluster on image-hash
+cl = Clustimage(method='phash', params_hash={'threshold':0, 'hash_size':32})
+
+# Example data
+X = cl.import_example(data='mnist')
+
+# Preprocessing, feature extraction and cluster evaluation
+results = cl.fit_transform(X, min_clust=4, max_clust=15, metric='euclidean', linkage='ward')
+
+# Scatter
+cl.scatter(zoom=3, img_mean=False, text=False)
+cl.scatter(zoom=None, img_mean=False, dotsize=20, text=False)
+cl.scatter(zoom=3, img_mean=False, text=True, plt_all=True, figsize=(35, 25))
+
+# cl.clusteval.plot()
+# cl.plot_unique(img_mean=False)
+# cl.plot(min_clust=5)
+
+
+# %%
+import pandas as pd
+df = pd.DataFrame(index=[model.results['filenames']], data=model.results['img_hash_hex'], columns=['image_hash_hex'])
+
+
 # %% Make plots in medium blog
 import cv2
 from scipy.spatial import distance
@@ -49,7 +77,7 @@ for method in methods:
     # Grayscaling and scaling
     model.import_data(targetdir)
     # Compute image for only the first image.
-    hashs = model.compute_hash(model.results['img'][0], to_array=True)
+    hashs = model.compute_imghash(model.results['img'][0], to_array=True)
     # Compute the image-hash
     print(method + ' Hash:')
     image_hash = ''.join(hashs[0].astype(int).astype(str).ravel())
@@ -70,24 +98,12 @@ for method in methods:
     ax[1][0].imshow(img_r, cmap='gray')
     ax[1][0].axis('off')
     ax[1][0].set_title('grayscale image, size %.0dx%.0d' %(8, 8))
-    ax[1][1].imshow(hashs[0], cmap='gray')
+    ax[1][1].imshow(hashs[0].reshape(8, 8), cmap='gray')
     ax[1][1].axis('off')
     ax[1][1].set_title(method + ' function')
 
     # Compute hash for the 10 images.
-    hashs = model.compute_hash(model, to_array=False)
-
-    # Compute average nr. of difference
-    adjmat = np.zeros((hashs.shape[0], hashs.shape[0]))
-    for i, h1 in enumerate(hashs):
-        for j, h2 in enumerate(hashs):
-            adjmat[i, j] = np.sum(h1!=h2)
-
-    diff=np.mean(adjmat[np.triu_indices(adjmat.shape[0], k=1)])
-    print('[%s] Average difference: %.2f' %(method, diff))
-
-    # Make a heatmap to demonstrate the differences between the image-hashes
-    imagesc.plot(hashs, cmap='gray', col_labels='', row_labels=model.results['filenames'], cbar=False, title=method + '\nAverage difference: %.3f' %(diff), annot=True)
+    hashs = model.compute_imghash(model, to_array=False)
 
 
 # %% Make plots in medium
@@ -108,7 +124,7 @@ model.import_data(targetdir)
 
 for method in methods:
     # Compute image-hash
-    model.fit_transform(method=method)
+    model.compute_hash(method=method)
 
     groups = []
     files = []
