@@ -8,9 +8,7 @@
 # --------------------------------------------------
 
 import os
-import pandas as pd
 import requests
-from urllib.parse import urlparse
 import logging
 import numpy as np
 from tqdm import tqdm
@@ -93,7 +91,7 @@ class Undouble():
     >>> model.import_data(targetdir)
     >>>
     >>> # Compute image-hash
-    >>> model.fit_transform()
+    >>> model.compute_hash()
     >>>
     >>> # Find images with image-hash <= threshold
     >>> model.group(threshold=0)
@@ -149,7 +147,7 @@ class Undouble():
         if return_results:
             return self.results
 
-    def fit_transform(self, method=None):
+    def compute_hash(self, method=None, hash_size=None):
         """Compute the hash for each image.
 
         Parameters
@@ -164,6 +162,8 @@ class Undouble():
                 * 'whash-db4': Daubechies wavelet hash
                 * 'colorhash': HSV color hash
                 * 'crop-resistant': Crop-resistant hash
+        hash_size : integer (default: 8)
+            The hash_size will be used to scale down the image and create a hash-image of length: hash_size*hash_size.
 
         Returns
         -------
@@ -174,12 +174,24 @@ class Undouble():
         if method is not None:
             self.params['method'] = method
             self.clustimage.params['method'] = method
-            self.clustimage.params_hash = cl.get_params_hash(method, {})
-        # Extract hash features
-        self.clustimage.extract_feat(self.results)
+        if hash_size is not None:
+            self.params['hash_size'] = hash_size
+            self.clustimage.params_hash['hash_size'] = hash_size
+        # Set hash parameters
+        self.clustimage.params_hash = cl.get_params_hash(self.params['method'], self.clustimage.params_hash)
+
+        # Compute image-hash features
+        self.results['img_hash_bin'] = np.array(list(map(self.clustimage.compute_hash, tqdm(self.results['img'], disable=disable_tqdm()))))
+        self.results['img_hash_hex'] = self.bin2hex()
+
+        # Build adjacency matrix for the image-hash based on nr. of differences
+        logger.info('Compute adjacency matrix [%gx%g] with absolute differences based on the image-hash of [%s].' %(self.results['img_hash_bin'].shape[0], self.results['img_hash_bin'].shape[0], self.params['method']))
+        self.results['adjmat'] = (self.results['img_hash_bin'][:, None, :] != self.results['img_hash_bin']).sum(2)
+
         # Remove keys that are not used.
         if 'labels' in self.results: self.results.pop('labels')
         if 'xycoord' in self.results: self.results.pop('xycoord')
+        if 'feat' in self.results: self.results.pop('feat')
 
     def group(self, threshold=0, return_dict=False):
         """Find similar images using the hash signatures.
@@ -194,8 +206,8 @@ class Undouble():
         None.
 
         """
-        if self.results['feat'] is None:
-            logger.warning('Can not group similar images because no features are present. Tip: Use the .fit_transform() function first.')
+        if self.results['img_hash_bin'] is None:
+            logger.warning('Can not group similar images because no features are present. Tip: Use the compute_hash() function first.')
             return None
 
         # Make sets of images that are similar based on the minimum defined threshold.
@@ -206,7 +218,7 @@ class Undouble():
         if np.sum(idx==False)>0:
             logger.warning('Files that are moved in a previous action are kept untouched.')
         paths = self.results['pathnames'][idx]
-        feat = copy.deepcopy(self.results['feat'])
+        feat = copy.deepcopy(self.results['adjmat'])
         feat = feat[idx, :]
         feat = feat[:, idx]
 
@@ -270,21 +282,20 @@ class Undouble():
         totfiles = np.sum(list(map(len, self.results['select_pathnames'])))
         totgroup = len(self.results['select_pathnames'])
         tdir = 'undouble' if targetdir is None else targetdir
-        answer = input('\n-------------------------------------------------\n>You are at the point of physically moving files.\n-------------------------------------------------\n>[%d] similar images are detected over [%d] groups.\n>[%d] images will be moved to the [%s] directory.\n>[%d] images will be copied to the [%s] directory.\n\n[C]ontinue moving all files.\n[W]ait in each directory.\n[Q]uit\nAnswer: ' %(totfiles, totgroup, totfiles - totgroup, tdir, totgroup, tdir))
+        answer = input('\n-------------------------------------------------\n>You are at the point of physically moving files.\n-------------------------------------------------\n>[%d] similar images are detected over [%d] groups.\n>[%d] images will be moved to the [%s] subdirectory.\n>[%d] images will be copied to the [%s] subdirectory.\n\n>[C]ontinue moving all files.\n>[W]ait in each directory.\n>[Q]uit\n>Answer: ' %(totfiles, totgroup, totfiles - totgroup, tdir, totgroup, tdir))
         answer = str.lower(answer)
         if answer == 'q':
             return
 
         # For each group, check the resolution and location.
         pathmem=''
-        # for pathnames in tqdm(self.results['select_pathnames'], disable=disable_tqdm()):
         for pathnames in self.results['select_pathnames']:
             curdir=os.path.split(pathnames[0])[0]
             if pathmem!=curdir:
                 pathmem=curdir
                 logger.info('Working in dir: [%s]' %(curdir))
                 if answer!='c':
-                    answer = input('><enter> to proceed to the next directory.\n>[C]ontinue to move all files.\n[Q]uit.')
+                    answer = input('><enter> to proceed to the next directory.\n>[C]ontinue to move all files.\n>[Q]uit\nAnswer: ')
                     answer = str.lower(answer)
                     if answer == 'q': return
             # Check file exists
@@ -345,7 +356,7 @@ class Undouble():
         Parameters
         ----------
         data : str
-            Name of datasets: 'flowers', 'faces', 'mnist', 'cat_and_dog'
+            Name of datasets: 'flowers', 'mnist', 'cat_and_dog'
         url : str
             url link to to dataset.
 
@@ -416,7 +427,7 @@ class Undouble():
         if not hasattr(self, 'results'):
             raise Exception(logger.error('Results missing! Hint: try to first use the model.group() functionality'))
 
-    def compute_hash(self, img, hash_size=None, to_array=False):
+    def compute_imghash(self, img, hash_size=None, to_array=False):
         """Compute hash.
 
         Parameters
@@ -437,9 +448,9 @@ class Undouble():
         """
         if hash_size is None: hash_size=self.params['hash_size']
         if hasattr(img, 'results'):
-            hashes = list(map(lambda x: self.clustimage.compute_hash(x, hash_size=hash_size).hash, self.results['img']))
+            hashes = list(map(lambda x: self.clustimage.compute_hash(x, hash_size=hash_size), self.results['img']))
         else:
-            hashes = self.clustimage.compute_hash(img, hash_size=hash_size).hash
+            hashes = self.clustimage.compute_hash(img, hash_size=hash_size)
             hashes = [hashes]
 
         # Convert to image-hash
@@ -449,6 +460,20 @@ class Undouble():
 
         return hashes
 
+    def bin2hex(self):
+        """Binary to hex.
+
+        Returns
+        -------
+        str
+            Hex of image hash.
+
+        """
+        if hasattr(self, 'results'):
+            return np.array(list(map(lambda x: hex(int(''.join(x.astype(int).astype(str)), 2)), self.results['img_hash_bin'])))
+        else:
+            logger.warning('Results missing! Hint: try to first use compute_hash()')
+
 
 # %% Import example dataset from github.
 def import_example(data='flowers', url=None):
@@ -591,6 +616,21 @@ def sort_images(pathnames, hash_scores=None, sort_first_img=False):
 
 # %%
 def filter_checks(pathnames, filters):
+    """Filter checks.
+
+    Parameters
+    ----------
+    pathnames : list of str
+        pathnames to the images.
+    filters : list, (Default: ['location'])
+        'location' : Only move images that are seen in the same directory.
+
+    Returns
+    -------
+    bool
+        When all filters are true.
+
+    """
     resOK, locOK = True, True
     # Check nr. of files
     fileOK = True if len(pathnames)>1 else False
@@ -742,6 +782,14 @@ def compute_blur(pathname):
 
 # %%
 def get_existing_pathnames(pathnames):
+    """Get existing pathnames.
+
+    Parameters
+    ----------
+    pathnames : list of str
+        pathnames to the images.
+
+    """
     Iloc = np.array(list(map(os.path.isfile, pathnames)))
     return pathnames[Iloc], np.array(Iloc)
 
@@ -760,6 +808,23 @@ def disable_tqdm():
 
 # %%
 def seperate_path(pathname):
+    """Seperate path.
+
+    Parameters
+    ----------
+    pathnames : list of str
+        pathnames to the images.
+
+    Returns
+    -------
+    dirname : str
+        directory path.
+    filename : str
+        filename.
+    ext
+        Extension.
+
+    """
     dirname, filename = os.path.split(pathname)
     filename, ext = os.path.splitext(filename)
     return dirname, filename, ext.lower()
@@ -770,5 +835,5 @@ def seperate_path(pathname):
 # if __name__ == "__main__":
 #     import undouble as undouble
 #     df = undouble.import_example()
-#     out = undouble.fit_transform(df)
+#     out = undouble.compute_hash(df)
 #     fig,ax = undouble.plot(out)
